#include <stdio.h>
#include <ctype.h>
#include <sys/types.h>
#include <fcntl.h>
#include <sys/mman.h>
#include <unistd.h>
#include <stdlib.h>

#ifndef MAP_FAILED
#define MAP_FAILED ((void *)-1)
#endif

int
main(int argc, char *argv[])
{
  int fd1, fd2, i;
  caddr_t pa1, pa2;
  int base, size, counter = 0;
  char buf[8192];

  if (argc != 5) {
    printf("usage: %s file1 file2 base size\n", argv[0]);
    exit(1);
  }
  fd1 = open(argv[1], O_RDWR);
  if (fd1 < 0) {
    perror(argv[1]);
    exit(1);
  }
  printf("opened file %s\n", argv[1]);

  fd2 = open(argv[2], O_RDWR);
  if (fd2 < 0) {
    perror(argv[2]);
    exit(1);
  }
  printf("opened file %s\n", argv[1]);

  base = atoi(argv[3]);
  size = atoi(argv[4]);
  printf("using base %d and size %d\n", base, size);

  pa1 = mmap((caddr_t) 0,
	     size,
	     (PROT_READ | PROT_WRITE),
	     MAP_SHARED,
	     fd1,
	     base);
  if (pa1 == MAP_FAILED) {
    perror("mmap1");
    exit(1);
  }
  printf("file1 is mapped with address 0x%x\n", (u_int) pa1);

  pa2 = mmap((caddr_t) 0,
	     size,
	     (PROT_READ | PROT_WRITE),
	     MAP_SHARED,
	     fd2,
	     base);
  if (pa2 == MAP_FAILED) {
    perror("mmap2");
    exit(1);
  }
  printf("file2 is mapped with address 0x%x\n", (u_int) pa2);

#if 0
  for (i = 0; i < SIZE; ++i) {
    printf("\tbyte %2d is %3d (%c)\n", i, pa[i],
	   isprint(pa[i]) ? pa[i] : '.');
  }

  /* hack to modify one byte in the mapped page */
  if (argc > 2) {
    printf("trying to modify the page...\n");
    pa[0] += 1;
  }
#endif

  /* compare bytes */
  for (i = 0; i < size; i++) {
    if (pa1[i] != pa2[i]) {
      if (counter < 10)
	printf("MISMATCH at offset %d (%d != %d)\n", i, pa1[i], pa2[i]);
      counter++;
    }
  }
  printf("There were %d mismatches\n", counter);

  i = read(fd2, buf, 8192);
  printf("read on fd2 returns %d\n", i);

  if (munmap(pa1, size) < 0) {
    perror("munmap1");
    exit(1);
  }
  printf("file1 is unmapped\n");

  if (munmap(pa2, size) < 0) {
    perror("munmap2");
    exit(1);
  }
  printf("file2 is unmapped\n");

  close(fd1);
  close(fd2);
  exit(0);
}
