/*
 * mapper4: copy file ARG1 to ARG2 using mmap.
 */
#include <stdio.h>
#include <ctype.h>
#include <sys/types.h>
#include <fcntl.h>
#include <sys/mman.h>
#include <sys/stat.h>
#include <unistd.h>

#ifndef MAP_FAILED
#define MAP_FAILED ((void *)-1)
#endif

int
main(int argc, char *argv[])
{
  int fd1, fd2, ret;
  caddr_t pa1;
  int base, size;
  struct stat statbuf;

  if (argc != 3) {
    printf("usage: %s src dst\n", argv[0]);
    exit(1);
  }
  /* get size of soure file */
  ret = stat(argv[1], &statbuf);
  if (ret < 0) {
    perror("stat");
    exit(1);
  }
  printf("source file size is %d\n", (u_int) statbuf.st_size);
  /* open source file */
  fd1 = open(argv[1], O_RDWR);
  if (fd1 < 0) {
    perror(argv[1]);
    exit(1);
  }
  printf("opened source file for read %s\n", argv[1]);

  /* unlink target file */
  unlink(argv[2]);
  printf("target file unlinked\n");

  /* open target file */
  fd2 = open(argv[2], O_RDWR|O_CREAT, 0664);
  if (fd2 < 0) {
    perror(argv[2]);
    exit(1);
  }
  printf("opened target file for read/write %s\n", argv[2]);

  base = 0;
  size = statbuf.st_size;
  printf("using base %d and size %d\n", base, size);

  pa1 = mmap((caddr_t) 0,
	     size,
	     (PROT_READ|PROT_WRITE),
	     MAP_SHARED,
	     fd1,
	     base);
  if (pa1 == MAP_FAILED) {
    perror("mmap1");
    exit(1);
  }
  printf("source file is mapped with address 0x%x\n", (u_int) pa1);

#if 0
  if (strcmp(argv[3], "y") == 0) {
    /* advise VM that source file will be read sequentially */
    madvise(pa1, size, MADV_SEQUENTIAL);
    printf("sequential VM advise done\n");
  }
#endif

  /* copy bytes! */
  printf("COPYING BYTES...\n");
  ret = write(fd2, pa1, size);
  if (ret != size) {
    printf("write error: returned %d\n", ret);
    exit(1);
  }
  printf("Copy done.\n");

  /* unmap mappings and close files */
  if (munmap(pa1, size) < 0) {
    perror("munmap1");
    exit(1);
  }
  printf("source file is unmapped\n");
  close(fd1);
  close(fd2);
  printf("source/target files closed\n");

  exit(0);
}
