#include <sys/types.h>
#include <sys/mman.h>
#include <sys/stat.h>
#include <errno.h>
#include <fcntl.h>
#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#define PAGESIZE 8192

#ifndef MAP_FAILED
#define MAP_FAILED ((void *)-1)
#endif

int
main(int argc, char *argv[])
{
  int fd1, i;
  caddr_t pa1;
  int base, size, counter = 0;
  char *buf, *mbuf = NULL;

  if (argc != 4) {
    printf("usage: %s file base size\n", argv[0]);
    exit(1);
  }
  base = atoi(argv[2]);
  size = atoi(argv[3]);
  printf("using base %d and size %d\n", base, size);

  /* allocate at least one more page */
  mbuf = malloc(size+PAGESIZE);
  if (!mbuf) {
    printf("no more memory\n");
    exit(1);
  }
  /* set buf to be page aligned into mbuf */
  buf = (char *) (((u_int) mbuf & ~(PAGESIZE-1)) + PAGESIZE);
  printf("mbuf=0x%x, buf=0x%x\n", (u_int)mbuf, (u_int)buf);

  fd1 = open(argv[1], O_RDWR);
  if (fd1 < 0) {
    perror(argv[1]);
    exit(1);
  }
  printf("opened file %s\n", argv[1]);

  pa1 = mmap(buf,
	     size,
	     (PROT_READ | PROT_WRITE),
	     (MAP_PRIVATE | MAP_FIXED), /* was: MAP_SHARED */
	     fd1,
	     base);
  if (pa1 == MAP_FAILED) {
    perror("mmap1");
    exit(1);
  }
  if (pa1 != buf) {
    perror("mmap private");
    exit(1);
  }
  printf("file1 is mapped with address 0x%x\n", (int) pa1);

  /* compare bytes */
  for (i = 0; i < size; i++) {
    if (pa1[i] == '\n') {
      counter++;
    }
  }
  printf("There were %d matches\n", counter);

  if (munmap(pa1, size) < 0) {
    perror("munmap1");
    exit(1);
  }
  printf("file1 is unmapped\n");

  close(fd1);

  exit(0);
}
